/*******************************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2013 Rob Withers
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *  ******************************************************************************
 * Murmur would not be possible without the ideas and implementation of 
 * the erights.org community, which is this software's foundation.  In particular, I 
 * would like to thank the following individuals:
 *         Mark Miller
 *         Marc Stiegler
 *         Bill Franz
 *         Tyler Close 
 *         Kevin Reid
 ******************************************************************************/
package cc.ganymede.pauwau.session.crypto;

import com.google.zxing.common.reedsolomon.GaloisField;
import com.google.zxing.common.reedsolomon.GenericGF;


public enum RSMode {
	TYPE_1() {
		public GaloisField getGaloisField() { return new GaloisField(GenericGF.OW_CODE_FIELD_16, 15, 9, 4); };
	},
	TYPE_2() {
		public GaloisField getGaloisField() { return new GaloisField(GenericGF.RS_256_A, 255, 247, 8); };
	},
	TYPE_3() {
		public GaloisField getGaloisField() { return new GaloisField(GenericGF.RS_256_B, 255, 223, 8); };
	};

	public static RSMode lookupRSMode(GenericGF galoisFieldImpl) {
		if(galoisFieldImpl.equals(GenericGF.OW_CODE_FIELD_16))
			return TYPE_1;
		if(galoisFieldImpl.equals(GenericGF.RS_256_A))
			return TYPE_2;
		if(galoisFieldImpl.equals(GenericGF.RS_256_B))
			return TYPE_3;
		return null;
	}

	public static RSMode valueOf(byte b) {
		for(RSMode rs:values()) {
			if(rs.ordinal() == b)
				return rs;
		}
		return null;
	}

	public abstract GaloisField getGaloisField();
}
