/*******************************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2013 Rob Withers
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *  ******************************************************************************
 * Murmur would not be possible without the ideas and implementation of 
 * the erights.org community, which is this software's foundation.  In particular, I 
 * would like to thank the following individuals:
 *         Mark Miller
 *         Marc Stiegler
 *         Bill Franz
 *         Tyler Close 
 *         Kevin Reid
 ******************************************************************************/
package cc.ganymede.pauwau.presentation.utils;

import static org.junit.Assert.*;

import org.apache.log4j.BasicConfigurator;
import org.junit.*;
import com.google.zxing.common.reedsolomon.GaloisField;
import com.google.zxing.common.reedsolomon.GenericGF;
import com.google.zxing.common.reedsolomon.ReedSolomonDecoder;
import com.google.zxing.common.reedsolomon.ReedSolomonEncoder;
import com.google.zxing.common.reedsolomon.ReedSolomonException;

public class EncodingTests {

	@Before
	public void setup() { BasicConfigurator.configure(); }

	@Test
	public void testBaseParametersForRS_15_9() {
		GaloisField galoisField = new GaloisField(GenericGF.OW_CODE_FIELD_16, 15, 9, 4);
		assertEquals(15, galoisField.getNumberOfChunkCodeSymbols());
		assertEquals(9, galoisField.getNumberOfChunkDataSymbols());
		assertEquals(30, galoisField.getNumberOfBlockCodeBytes());
		assertEquals(18, galoisField.getNumberOfBlockDataBytes());
		assertEquals(60, galoisField.getNumberOfBlockCodeSymbols());
		assertEquals(36, galoisField.getNumberOfBlockDataSymbols());
		assertEquals(6, galoisField.getECSymbols());
		assertEquals(4, galoisField.getSymbolSizeInBits());
	}

	@Test
	public void testBaseParametersForRS_255_247() {
		GaloisField galoisField = new GaloisField(GenericGF.RS_256_A, 255, 247, 8);
		assertEquals(255, galoisField.getNumberOfChunkCodeSymbols());
		assertEquals(247, galoisField.getNumberOfChunkDataSymbols());
		assertEquals(1020, galoisField.getNumberOfBlockCodeBytes());
		assertEquals(988, galoisField.getNumberOfBlockDataBytes());
		assertEquals(1020, galoisField.getNumberOfBlockCodeSymbols());
		assertEquals(988, galoisField.getNumberOfBlockDataSymbols());
		assertEquals(8, galoisField.getECSymbols());
		assertEquals(8, galoisField.getSymbolSizeInBits());
	}

	@Test
	public void testBaseParametersForRS_255_223() {
		GaloisField galoisField = new GaloisField(GenericGF.RS_256_B, 255, 223, 8);
		assertEquals(255, galoisField.getNumberOfChunkCodeSymbols());
		assertEquals(223, galoisField.getNumberOfChunkDataSymbols());
		assertEquals(1020, galoisField.getNumberOfBlockCodeBytes());
		assertEquals(892, galoisField.getNumberOfBlockDataBytes());
		assertEquals(1020, galoisField.getNumberOfBlockCodeSymbols());
		assertEquals(892, galoisField.getNumberOfBlockDataSymbols());
		assertEquals(32, galoisField.getECSymbols());
		assertEquals(8, galoisField.getSymbolSizeInBits());
	}

	@Test
	public void testEncodeAndDecodeRS_15_9() throws ReedSolomonException {
		GaloisField galoisField = new GaloisField(GenericGF.OW_CODE_FIELD_16, 15, 9, 4);
		byte[] chunk = new byte[] { 
				(byte) 0x0F, 
				(byte) 0x0F, 
				(byte) 0x0A, 
				(byte) 0x0A, 
				(byte) 0x0B, 
				(byte) 0x0B, 
				(byte) 0x09, 
				(byte) 0x09, 
				(byte) 0x0D};
		byte[] encodedBytes = new ReedSolomonEncoder(galoisField).encodeBytes(chunk);
		byte[] decodedBytes = new ReedSolomonDecoder(galoisField).decodeBytes(encodedBytes);
		assertArrayEquals(chunk, decodedBytes);
	}
	@Test
	public void testEncodeAndDecodeRS_255_247() throws ReedSolomonException {
		GaloisField galoisField = new GaloisField(GenericGF.RS_256_A, 255, 247, 8);
		byte[] chunk = new byte[247];
		byte[] test = new byte[] { 
				(byte) 0x0F, 
				(byte) 0x0E, 
				(byte) 0x0D, 
				(byte) 0x0B, 
				(byte) 0x0A, 
				(byte) 0x09, 
				(byte) 0x08, 
				(byte) 0x07, 
				(byte) 0x06, 
				(byte) 0x05};
		for(int i = 0; i < 24; i++) {
			System.arraycopy(test, 0, chunk, i*10, test.length);
		}
		test = new byte[] { 
				(byte) 0x0F, 
				(byte) 0x0E, 
				(byte) 0x0D, 
				(byte) 0x0B, 
				(byte) 0x0A,
				(byte) 0x09,
				(byte) 0x08 };
		System.arraycopy(test, 0, chunk, 240, test.length);
		byte[] encodedBytes = new ReedSolomonEncoder(galoisField).encodeBytes(chunk);
		byte[] decodedBytes = new ReedSolomonDecoder(galoisField).decodeBytes(encodedBytes);
		assertArrayEquals(chunk, decodedBytes);
	}
	@Test
	public void testEncodeAndDecodeRS_255_223() throws ReedSolomonException {
		GaloisField galoisField = new GaloisField(GenericGF.RS_256_B, 255, 223, 8);
		byte[] chunk = new byte[223];
		byte[] test = new byte[] { 
				(byte) 0x0F, 
				(byte) 0x0E, 
				(byte) 0x0D, 
				(byte) 0x0B, 
				(byte) 0x0A, 
				(byte) 0x09, 
				(byte) 0x08, 
				(byte) 0x07, 
				(byte) 0x06, 
				(byte) 0x05};
		for(int i = 0; i < 22; i++) {
			System.arraycopy(test, 0, chunk, i*10, test.length);
		}
		test = new byte[] { 
				(byte) 0x0F, 
				(byte) 0x0E, 
				(byte) 0x0D };
		System.arraycopy(test, 0, chunk, 220, test.length);
		byte[] encodedBytes = new ReedSolomonEncoder(galoisField).encodeBytes(chunk);
		byte[] decodedBytes = new ReedSolomonDecoder(galoisField).decodeBytes(encodedBytes);
		assertArrayEquals(chunk, decodedBytes);
	}
}
